"""
scrapple.selectors.css
~~~~~~~~~~~~~~~~~~~~~~

Defines the CSS selector
"""

from lxml import cssselect

from scrapple.selectors.selector import Selector


class CssSelector(Selector):
	"""
	CssSelector defines the CSS selector
	"""
	
	def __init__(self, url):
		super(CssSelector, self).__init__(url)


	def extract_content(self, selector, attr):
		"""
		Method for performing the content extraction for the given CSS selector.
		"""
		try:
			sel = cssselect.CSSSelector(selector)
			if attr == "text":
				content = "".join([x.text for x in sel(self.tree)])
			else:
				content = sel(self.tree)[0].get(attr)
			return content
		except IndexError:
			raise Exception("There is no content for the selector " + selector)


	def extract_links(self):
		"""
		Method for performing the link extraction for the crawler.
		"""
		raise NotImplementedError


	def extract_tabular(self):
		"""
		Method for performing the content extraction from tabular organized data.
		"""
		raise NotImplementedError



"""
Scrapple
========

Scrapple is a framework for building and running web scrapers/crawlers according to \
key-value pair based configuration files.

:copyright: 2015 Alex Mathew, see AUTHORS for more details
:license: MIT, see LICENSE for more details
"""

from __future__ import print_function

__author__ = "Alex Mathew, and Harish Balakrishnan"
__copyright__ = "Copyright 2015, Alex Mathew"
__credits__ = ["Alex Mathew", "Harish Balakrishnan"]
__license__ = "MIT"
__version__ = "0.1"
__status__ = "Development"
__maintainer__ = "Alex Mathew"
__email__ = "alexmathew003@gmail.com"

__all__ = ["__version__", "command"]

import sys
if sys.version_info < (2, 7):
	print("Scrapple %s requires atleast Python 2.7" % __version__)
	sys.exit(1)
del sys



"""
scrapple.commands.genconfig
~~~~~~~~~~~~~~~~~~~~~~~~~~~

"""

from __future__ import print_function
from jinja2 import Template
import os
import json
from colorama import init, Fore, Back

import scrapple
from scrapple.commands import command

class GenconfigCommand(command.Command):
    """
    Defines the execution of :command: genconfig
    """

    def __init__(self, args):
        self.args = args
        init()

    def execute_command(self):
        """
        Execution method of :command: genconfig
        """
        print(Back.GREEN + Fore.BLACK + "Scrapple Genconfig")
        print(Back.RESET + Fore.RESET)
        directory = os.path.join(scrapple.__path__[0], 'templates', 'configs')
        with open(os.path.join(directory, self.args['--type'] + '.txt'), 'r') as f:
            template_content = f.read()
        print("\n\nUsing the", self.args['--type'], "template\n\n")
        template = Template(template_content)
        settings = {
            'projectname': self.args['<projectname>'],
            'selector_type': self.args['--selector'],
            'url': self.args['<url>']
        }
        rendered = template.render(settings=settings)
        json_content = eval(rendered)
        with open(self.args['<projectname>'] + '.json', 'w') as f:
            json.dump(json_content, f)
        print(Back.WHITE + Fore.RED + self.args['<projectname>'], ".json has been created" \
            + Back.RESET + Fore.RESET, sep="")



"""
scrapple.command
~~~~~~~~~~~~~~~~

Contains the definition for the ``Command`` class which provides the basic description of \
the commands used with Scrapple
"""

class Command(object):
    """
    Creates a ``Command`` object, that provides a description of a Scrapple command
    """
    
    def __init__(self, args):
        self.args = args

    def execute_command(self):
        """
        Function for executing the specified command
        """
        raise NotImplementedError



"""
scrapple.selectors.xpath
~~~~~~~~~~~~~~~~~~~~~~~~

Defines the XPath selector
"""

from scrapple.selectors.selector import Selector


class XpathSelector(Selector):
	"""
	XpathSelector defines the XPath selector
	"""
	
	def __init__(self, url):
		self.url = url
		super(XpathSelector, self).__init__(url)


	def extract_content(self, selector, attr):
		"""
		Method for performing the content extraction for the given XPath expression.
		"""
		try:
			if attr == "text":
				content = "".join([x.text for x in self.tree.xpath(selector)])
			else:
				content = self.tree.xpath(selector)[0].get(attr)
			return content
		except IndexError:
			raise Exception("There is no content for the selector " + selector)


	def extract_links(self):
		"""
		Method for performing the link extraction for the crawler.
		"""
		raise NotImplementedError


	def extract_tabular(self):
		"""
		Method for performing the content extraction from tabular organized data.
		"""
		raise NotImplementedError



"""
scrapple.commands.run
~~~~~~~~~~~~~~~~~~~~~

"""

from __future__ import print_function
import os
import json
from colorama import init, Fore, Back

from scrapple.commands import command
from scrapple.selectors import xpath, css

class RunCommand(command.Command):
    """
    Defines the execution of :command: run
    """

    def __init__(self, args):
        self.args = args
        init()

    def execute_command(self):
        """
        Execution method of :command: run
        """
        print(Back.GREEN + Fore.BLACK + "Scrapple Run")
        print(Back.RESET + Fore.RESET)
        try:
            with open(self.args['<projectname>'] + '.json', 'r') as f:
                self.config = json.load(f)
            runner = getattr(self, self.args['--type'])
            runner()
        except IOError:
            print(Back.WHITE + Fore.RED + self.args['<projectname>'], ".json does not ", \
                  "exist. Use ``scrapple genconfig``." + Back.RESET + Fore.RESET, sep="")


    def scraper(self):
        selectorClass = getattr(
                eval(self.config['selector_type']), 
                self.config['selector_type'].title() + 'Selector'
                )
        results = dict()
        try:
            print()
            print(Back.YELLOW + Fore.BLUE + "Loading page ", self.config['scraping']['url'] \
                + Back.RESET + Fore.RESET)
            selector = selectorClass(self.config['scraping']['url'])
            for attribute in self.config['scraping']['data']:
                if attribute['field'] != "":
                    print("\nExtracting", attribute['field'], "attribute", sep=' ')
                    results[attribute['field']] = selector.extract_content(attribute['selector'], attribute['attr'])
        except Exception, e:
            print(e)
        finally:
            with open(os.path.join(os.getcwd(), self.args['<output_filename>'] + '.json'), 'w') as f:
                json.dump(results, f)
            print()
            print(Back.WHITE + Fore.RED + self.args['<output_filename>'], \
                  ".json has been created" + Back.RESET + Fore.RESET, sep="")


    def crawler(self):
        selectorClass = getattr(
                eval(self.config['selector_type']), 
                self.config['selector_type'].title() + 'Selector'
                )
        selector = selectorClass(self.config['scraping']['url'])
        return



"""
scrapple.utils.form
~~~~~~~~~~~~~~~~~~~

Functions related to form handling.
"""

import os
import json
import itertools


def form_to_json(form):
	config = dict()
	if form['project_name'] == "":
		raise Exception('Project name cannot be empty.')
	if form['selector_type'] not in ["css", "xpath"]:
		raise Exception('Selector type has to css or xpath')
	config['project_name'] = form['project_name']
	config['selector_type'] = form['selector_type']
	config['scraping'] = dict()
	if form['url'] == "":
		raise Exception('URL cannot be empty')
	config['scraping']['url'] = form['url']
	config['scraping']['data'] = list()
	for i in itertools.count(start=1):
		try:
			data = {
				'field': form['field_' + str(i)],
				'selector': form['selector_' + str(i)],
				'attr': form['attribute_' + str(i)]
			}
			config['scraping']['data'].append(data)
		except KeyError:
			break
	# TODO : Crawler 'next' parameter handling
	with open(os.path.join(os.getcwd(), form['project_name'] + '.json'), 'w') as f:
		json.dump(config, f)
	return



"""
scrapple.utils.selector
~~~~~~~~~~~~~~~~~~~~~~~

Defines Scrapple selectors
"""

import requests

class ScrappleSelector(object):
	"""
	ScrappleSelector objects allow using CSS selectors or XPath expressions \
	as needed by the user. 
	"""
	
	def __init__(self, url, selector):
		content = requests.get(url).content



"""
scrapple.utils.dynamicdispatch
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

Functions related to dynamic dispatch of objects
"""

def get_command_class(command):
    from scrapple.commands import genconfig, generate, run, web
    cmdClass = getattr(eval(command), command.title() + 'Command')
    return cmdClass
